package core

const (
	IPv6HasNoBroadcastAddressError = "IPv6 network has no broadcast addresses"
	IPv4HasNoBroadcastAddressError = "IPv4 network has no broadcast address"

	IPv4NetworkHasNoFirstUsableAddressError = "IPv4 network has no first usable address"
	IPv6NetworkHasNoFirstUsableAddressError = "IPv6 network has no first usable address"

	IPv4NetworkHasNoLastUsableAddressError = "IPv4 network has no last usable address"
	IPv6NetworkHasNoLastUsableAddressError = "IPv6 network has no last usable address"
)
